@@ -4,7 +4,8 @@ import sys
 import os
 
 from prophyc import options
-from prophyc import model_sort
+
+__version__ = '0.4.1'
 
 def get_basename(filename):
     return os.path.splitext(os.path.basename(filename))[0]
@@ -20,17 +21,27 @@ def module_exists(module_name):
 def main():
     opts = options.parse_options()
 
+    if opts.version:
+        print "prophyc {}".format(__version__)
+        sys.exit(0)
+
+    if not opts.input_files:
+        sys.exit("prophyc: error: missing input file")
+
     if opts.isar:
         from prophyc.parsers.isar import IsarParser
         parser = IsarParser()
+        parse_error = None
     elif opts.sack:
         if not module_exists("clang"):
             sys.exit("Sack input requires clang and it's not installed")
         from prophyc.parsers.sack import SackParser
         parser = SackParser(opts.include_dirs)
+        parse_error = None
     else:
-        from prophyc.parsers.prophy import ProphyParser
+        from prophyc.parsers.prophy import ProphyParser, ParseError
         parser = ProphyParser()
+        parse_error = ParseError
 
     serializers = []
     if opts.python_out:
@@ -44,15 +55,21 @@ def main():
         sys.exit("Missing output directives")
 
     for input_file in opts.input_files:
-        basename = get_basename(input_file)
-        nodes = parser.parse(input_file)
+        try:
+            nodes = parser.parse(input_file)
+        except parse_error as e:
+            sys.exit('\n'.join(e.errors))
+
         if opts.patch:
             from prophyc import patch
             patches = patch.parse(opts.patch)
             patch.patch(nodes, patches)
+        model.topological_sort(nodes)
+        model.cross_reference(nodes)
+        model.evaluate_kinds(nodes)
 
-        model_sort.model_sort(nodes)
         for serializer in serializers:
+            basename = get_basename(input_file)
             serializer.serialize(nodes, basename)
 
 if __name__ == "__main__":
